@@ -24,9 +24,9 @@ typedef unsigned int word;
 typedef unsigned long dword,d_word;
 typedef int Boolean;
 #ifdef SABREWIN
-#define true 1
-#define false 0
 #else
+#ifndef USES_DDRAW
 typedef int boolean;
 #endif
 #endif
+#endif
