@@ -1,5 +1,5 @@
 /*
-    SABRE Fighter Plane Simulator 
+    Sabre Fighter Plane Simulator 
     Copyright (c) 1997 Dan Hammer
     Portions Donated By Antti Barck
 
@@ -18,7 +18,7 @@
     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
 /*************************************************
- *           SABRE Fighter Plane Simulator       *
+ *           Sabre Fighter Plane Simulator       *
  * File   : fltobj.h                             *
  * Date   : February, 1998                       *
  * Author : Dan Hammer                           *
@@ -29,6 +29,7 @@
 #define __fltobj_h
 
 #include "dvmath.h"
+#include "spid.h"
 
 class C_3DObject_Base;
 extern float time_frame;
@@ -58,9 +59,18 @@ extern float world_scale;
 #define FLAPS_STEP       5
 #define TRIM_MIN       -30
 #define TRIM_MAX        30
+
 /* enums for view */
-typedef enum flight_view { front, left, right, back, satellite,
-			   outside_1, outside_2, outside_3, outside_4 };
+typedef enum flight_view {
+				fv_front, fv_left, fv_right, fv_rear, fv_satellite,
+			   fv_external, fv_track, fv_target_track, fv_flyby,
+				fv_front_up, fv_front_down, fv_left_up, fv_left_down, fv_right_up,
+				fv_right_down, fv_rear_up, fv_rear_down, fv_up, fv_down,
+				fv_weapon_track, fv_virtual, fv_front_left, fv_front_left_up,
+				fv_front_left_down, fv_front_right, fv_front_right_up, fv_front_right_down,
+				fv_rear_left, fv_rear_left_up, fv_rear_left_down, fv_rear_right,
+				fv_rear_right_up, fv_rear_right_down, fv_padlock
+};
 
 /******************************************************************
  * specifications class                                           *
@@ -76,7 +86,7 @@ public:
   float drag_factor;                   /* drag coefficient */
   float idrag_factor;                  /* induced drag coefficient */
   float lift_factor;                   /* lift coefficient */
-  float wing_aoa;	               /* angle-of-attack of wing */
+  float wing_aoa;                      /* angle-of-attack of wing */
   float max_aoa;                       /* maximum angle-of-attack before stall */
   float max_aoa_factor;                /* max_aoa before stall relative to velocity */
   /*****************************
@@ -95,7 +105,7 @@ public:
   float control_yaw;                   /* yawing moment produced by rudder position */
   float control_roll;	               /* rolling moment produced by aileron positions */
   float return_pitch;	               /* pitching moment towards flight path produced by horiz. stab. */
-  float return_yaw;		       /* yawing moment towards flight path produced by vert. stab. */ 
+  float return_yaw;		               /* yawing moment towards flight path produced by vert. stab. */ 
   float pitch_drag;                    /* drag factor for pitching */
   float yaw_drag;                      /* drag factor for yawing */
   float roll_drag;                     /* drag factor for rolling */
@@ -118,65 +128,45 @@ public:
    * more aeorodynamically    *
    * suspicious stuff         *
    ****************************/
-  float g_height;                      /* height of fuselage when on ground w gear down */
-  float flap_lift;   		       /* how much flaps contribute to lift */
-  float flap_drag;		       /* how much flaps contribute to drag */
+  float g_height;           /* height of fuselage when on ground w gear down */
+  float flap_lift;   		 /* how much flaps contribute to lift */
+  float flap_drag;		    /* how much flaps contribute to drag */
   float fuel_cap;		       /* capacity in lbs of internal fuel tank */
   float wb_damp;		       /* damping factor for wheel brakes */
-  float load_limit;		       /* limit on load (lift / weight) */
-  float stall_spin;		       /* rolling moment from stall */
+  float load_limit;		    /* limit on load (lift / weight) */
+  float stall_spin;		    /* rolling moment from stall */
   int max_damage;		       /* maximum amount of abuse the poor plane can stand */
-  R_3DPoint view_point;		       /* pilot's head position in cockpit ("port" coords") */
-  int engine_type;		       /* 0 = prop, 1 = jet, 2 = jet with afterburner */
+  R_3DPoint view_point;		 /* pilot's head position in cockpit ("port" coords") */
+  int engine_type;		    /* 0 = prop, 1 = jet, 2 = jet with afterburner */
   long flags;			       /* general-purpose flags */
-  char model[32];                      /* model */
+  char model[32];           /* model */
+
+  /**************************
+   * AND MORE! AND MORE!    *
+	**************************/
+  _sPID pitchPid;					/* pid for pitch control */
+  _sPID rollPid;					/*         roll control  */
+  _sPID yawPid;               /*         yaw control   */
+  _sPID gPid;                 /*         g's control   */
+  _sPID airSpeedPid;          /*         air speed     */
+  _sPID targetClosurePid;     /*         target closure */
+
+  /*************************
+   * when will it end???   *
+	*************************/
+  float formationWingLen;	   /* wing length for formation spacing */
+  float formationOffset;		/* distance to align point for formation */
+
+  /************************************
+   * Contributed by Pierre de Moveaux *
+   * Dihedral effect parameters       *
+   ************************************/
+  float dihedralX;
+  float dihedralY;
+  float dihedralZ;
+
+  Flight_Specs();
 
-  Flight_Specs()
-  {
-    weight = 0.0;
-    max_thrust = 0.0;
-    max_speed = 0.0;
-    corner_speed = 0.0;
-    drag_factor = 0.0;
-    idrag_factor = 0.0;
-    lift_factor = 0.0;
-    wing_aoa = 0.0;
-    max_aoa = 0.0;
-    drag_aoa = 0.0;
-    drag_yaw = 0.0;
-    drag_wb = 0.0;
-    drag_gr = 0.0;
-    drag_sb = 0.0;
-    drag_whls = 0.0;
-    control_pitch = 0.0;
-    control_yaw = 0.0;
-    control_roll = 0.0;
-    return_pitch = 0.0;
-    return_yaw = 0.0;
-    pitch_drag = 0.0;
-    yaw_drag = 0.0;
-    roll_drag = 0.0;
-    pitch_damp = 0.0;
-    yaw_damp = 0.0;
-    roll_damp = 0.0;
-    lspeed = 0.0;
-    l_z = 0.0;
-    max_wb = 0.0;
-    l_aoa_max = 0.0;
-    adv_yaw = 0.0;
-    adv_roll = 0.0;
-    g_height = 0.0;
-    flap_lift = 0.01;
-    flap_drag = 0.003;
-    fuel_cap = 100.0;
-    wb_damp = 2.0;
-    load_limit = 9.9;
-    stall_spin = 0.5;
-    max_damage = 100;
-    max_aoa_factor = 0.03;
-    engine_type = JET_TYPE;
-    flags = 0;
-  }
   void read(istream &);
   void write(ostream &);
   friend istream &operator >>(istream &is, Flight_Specs &fs)
@@ -226,7 +216,7 @@ public:
   int autopilot;
   int radar;
   int armed_w;                    /* arm weapon */
-
+  int auto_coord;						 /* no adverse yaw, roll */
   int a_burner;                   /* after burner stage */
   int s_weapon;                   /* selected weapon index */
 
@@ -236,10 +226,17 @@ public:
   int hud_on;                     /* display hud */
   int vect_on;                    /* display flight-path vector */
   int show_controls;              /* display yoke position, rudder position etc */
+  /* for new ai -- display maneuver stack & target flags */
+  int show_ai_info;
   /* outside viewing params */
+  R_3DPoint vPoint;					 /* view position in world coords */
   REAL_TYPE vdist;                /* distance from aircraft */                 
   REAL_TYPE vtheta;               /* horiz. rotation */
   REAL_TYPE vphi;                 /* vert. rotation */
+  /* virtual cockpit params */
+  REAL_TYPE virtTheta;				 /* horiz rot from front */
+  REAL_TYPE virtPhi;					 /* vert rot from front */
+  REAL_TYPE virtStep;				 /* rotation step */
   /* current view */
   int view;
 
@@ -266,7 +263,7 @@ public:
     trim = 0;
     wheel_brakes = 0;
     landing_gear = 0;
-    view = front;
+    view = fv_front;
     engine_on = 0;
     bang_bang = 0;
     autopilot = 0;
@@ -281,8 +278,14 @@ public:
     vdist = 100;
     vtheta = 0.6;
     vphi = 0.2;
+	 virtTheta = 0.0;
+	 virtPhi = 0.0;
+	 virtStep = 0.1;
     show_controls = 1;
+	 show_ai_info = 0;
+	 auto_coord = 0;
   }
+
   void read(istream &is);
   void write(ostream &os);
   friend istream & operator >>(istream &is, Flight_Controls &fc)
@@ -343,15 +346,15 @@ public:
 class Flight_Forces
 {
 public:
-  /* "Vector" forces */
-  Vector_Q thrust;			  /* thrust of engine */
-  Vector_Q drag;	                  /* opposing drag */
-  Vector_Q lift;                          /* lift */
-  Vector_Q g;                             /* weight */
-  /* rotational accelerations */
-  float pitch_acc;                        /* net pitching acceleration */
-  float yaw_acc;                          /* net yawing acceleration */
-  float roll_acc;                         /* net rolling acceleration */
+  /* "Vector" forces                                      */
+  Vector_Q thrust; /* thrust of engine                    */
+  Vector_Q drag;   /* opposing drag                       */
+  Vector_Q lift;   /* lift                                */
+  Vector_Q g;      /* weight                              */
+  /* rotational accelerations                             */
+  float pitch_acc; /* net pitching acceleration           */
+  float yaw_acc;   /* net yawing acceleration             */
+  float roll_acc;  /* net rolling acceleration            */
   Flight_Forces()
   {
     pitch_acc = yaw_acc = roll_acc = 0.0;
@@ -365,49 +368,49 @@ class Flight_State
 {
 public:
   Vector_Q velocity;		 /* velocity in feet-per-second, in world coordinants */
-  DVector vel_pv;		 /* direction of velocity in port coordinants */
-  float z_vel;			 /* magnitude of velocity along z-axis ("air speed") */
-  float delta_z_vel;		 /* rate of change in above */
-  float climb_vel;               /* climb rate in feet-per-second */
-  float delta_climb_vel;         /* rate of change of rate of climb (whew!) */
-  float angle_of_attack;         /* also known as "angle of incidence", the flight-path v fuselage */
-  float yaw;			 /* actually, the cos of the yaw (1 ... -1) */
-  DVector h_direction;		 /* Vector representing movement on xy plane */
-  float heading;		 /* the heading for above, in radians */
-  Port_3D flight_port;		 /* gives position and orientation */
-  int inverted;			 /* true if inverted */
-  int negative_roll;		 /* true if rolled into negative angle */
-  int negative_phi;              /* true if pitched into negative angle */
-  int crashed;			 /* true if aerodynamically challenged */
-  int stalled;			 /* true if stalled */
-  float near_stall;		 /* how far from stall angle (radians) */
-  float pitch_rate;		 /* angular rate of pitch (radians per second) */
-  float d_pitch_rate;            /* rate of change for above */
-  float yaw_rate;		 /* angular rate of yaw (radians per second) */
-  float d_yaw_rate;              /* rate of change for above */
+  DVector vel_pv;        /* direction of velocity in port coordinants */
+  float z_vel;           /* magnitude of velocity along z-axis ("air speed") */
+  float delta_z_vel;     /* rate of change in above */
+  float climb_vel;       /* climb rate in feet-per-second */
+  float delta_climb_vel; /* rate of change of rate of climb (whew!) */
+  float angle_of_attack; /* also known as "angle of incidence", the flight-path v fuselage */
+  float yaw;             /* actually, the cos of the yaw (1 ... -1) */
+  DVector h_direction;   /* Vector representing movement on xy plane */
+  float heading;         /* the heading for above, in radians */
+  Port_3D flight_port;   /* gives position and orientation */
+  int inverted;          /* true if inverted */
+  int negative_roll;     /* true if rolled into negative angle */
+  int negative_phi;      /* true if pitched into negative angle */
+  int crashed;           /* true if aerodynamically challenged */
+  int stalled;           /* true if stalled */
+  float near_stall;      /* how far from stall angle (radians) */
+  float pitch_rate;      /* angular rate of pitch (radians per second) */
+  float d_pitch_rate;    /* rate of change for above */
+  float yaw_rate;        /* angular rate of yaw (radians per second) */
+  float d_yaw_rate;      /* rate of change for above */
   float roll_rate;		 /* angular rate of roll (radians per second) */
-  float d_roll_rate;             /* rate of change for above */
-  int on_ground;		 /* true if on ground */
+  float d_roll_rate;     /* rate of change for above */
+  int on_ground;         /* true if on ground */
   /* landing parameters */
-  float landing_z;		 /* vertical velocity upon landing */
-  float landing_velocity;	 /* speed upon landing */
+  float landing_z;       /* vertical velocity upon landing */
+  float landing_velocity;/* speed upon landing */
   float landing_aoa;		 /* aoa on landing */
-  float landing_pitch;           /* pitch relative to ground on landing */
-  float landing_roll;		 /* roll on landing */
-  int was_airborne;  		 /* flag indicating we were at one time airborne */
-  int landing_recorded;		 /* landing data recorded */
+  float landing_pitch;   /* pitch relative to ground on landing */
+  float landing_roll;    /* roll on landing */
+  int was_airborne;      /* flag indicating we were at one time airborne */
+  int landing_recorded;  /* landing data recorded */
   /* more stuff */
-  float load;		         /* load (lift / weight); */
-  float d_load;                  /* change in */
+  float load;            /* load (lift / weight); */
+  float d_load;          /* change in */
   float air_density;		 /* air density factor */
   float vel_factor;		 /* velocity factor in determining roll rates */
-  float ground_height;           /* height-of-terrain below us */
-  float d_ground_height;	 /* change in */
-  float agl;                     /* Altitude above Ground Level */
-  float d_agl;                   /* change in */
-  float weight;                  /* current weight */
-  float fuel;                    /* current lbs of fuel */
-  float elapsed_time;            /* elapsed time in seconds */
+  float ground_height;   /* height-of-terrain below us */
+  float d_ground_height; /* change in */
+  float agl;             /* Altitude above Ground Level */
+  float d_agl;           /* change in */
+  float weight;          /* current weight */
+  float fuel;            /* current lbs of fuel */
+  float elapsed_time;    /* elapsed time in seconds */
   Flight_State()
   {
     z_vel = 0.0;
@@ -508,4 +511,10 @@ public:
   }
 };
 
+inline int IS_COCKPIT_VIEW(int viewType)
+{
+	return viewType <= fv_rear ||
+			 (viewType >= fv_front_up && viewType <= fv_rear_down) ||
+			 (viewType >= fv_virtual && viewType <= fv_padlock);
+}
 #endif
