@@ -1,6 +1,6 @@
 /*
-    SABRE Fighter Plane Simulator 
-    Copyright (c) 1997 Dan Hammer
+    Sabre Fighter Plane Simulator 
+    Copyright (c) 1997/98 Dan Hammer
     Portions Donated By Antti Barck
 
     This program is free software; you can redistribute it and/or modify
@@ -18,7 +18,7 @@
     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
 /*************************************************
- *           SABRE Fighter Plane Simulator       *
+ *           Sabre Fighter Plane Simulator       *
  * File   : grndlevl.h                           *
  * Date   : March, 1997                          *
  * Author : Dan Hammer                           *
