@@ -1,6 +1,6 @@
 /*
-    SABRE Fighter Plane Simulator 
-    Copyright (c) 1997 Dan Hammer
+    Sabre Fighter Plane Simulator 
+    Copyright (c) 1997/98 Dan Hammer
     Portions Donated By Antti Barck
 
     This program is free software; you can redistribute it and/or modify
@@ -31,17 +31,21 @@ class KBHit
 public:
   fd_set rfds;
   struct termios oldios,newios;
+  static int kbdin;
 
   KBHit();
   ~KBHit();
   int kbdhit();
   int getch();
+  int GetKbdin()
+    {
+      return kbdin;
+    }
+
 };
 extern KBHit kbhit;
 
 #endif
 
-extern int kbdin;
-
 #endif
 
