@@ -1,6 +1,6 @@
 /*
-    SABRE Fighter Plane Simulator 
-    Copyright (c) 1997 Dan Hammer
+    Sabre Fighter Plane Simulator 
+    Copyright (c) 1997/98 Dan Hammer
     Portions Donated By Antti Barck
 
     This program is free software; you can redistribute it and/or modify
@@ -48,6 +48,7 @@ public:
   MTimer();
   float get_time();
   int check_time();
+  void reset();
 };
 
 
